<?php
/**
 * sys_config.php
 * 负责系统选项的UI工作
 * 支持功能包括
 * 1. 系统配置各个参数的合法性检测
 * 2. 获取所有系统配置数据
 * 3. 测试邮件有效性
 * 4. 更改系统配置
 */
require_once(ldb_ext_root()."/bin/mapreduce/client.php");
require_once(ldb_ext_root().'/bin/misc/mail/class.smtp.php');
require_once(ldb_ext_root().'/bin/misc/mail/class.phpmailer.php');

call_user_func(function () {
    // 返回的密码常数
    $pwd_const = "fixpwd135792468";
    /**
     * web端口更改，如果端口更改，执行操作待定
     * @param int $new_http_port 新的web端口
     */
    $change_http_port = function ($old_http_port, $new_http_port) {
        // 比较web端口是否更改
        if ((strcmp($old_http_port, $new_http_port) == 0)) {
            return -1;
        }
        $file_name = ldb_ext_root()."../nginx/conf/nginx.conf";
        $file_data = file_get_contents($file_name);
        $conf_data = explode("\n", $file_data);
        foreach ($conf_data as $key => $value) {
            if (preg_match("/^[\s]+listen/", $value) != 0) {
                $conf_data[$key] = "        listen ".$new_http_port." ssl;";
                continue;
            }
        }
        $file_data = "";
        foreach ($conf_data as $line) {
            $file_data .= $line."\r\n";
        }
        $tmp_name = ldb_get_temp_dir()."/httpd-ssl_tmp.js"; // 临时文件
        if (!file_put_contents($tmp_name, $file_data)
            || !rename($tmp_name, $file_name)) {
            unlink($tmp_name);
            return -1;
        }
        return 0;
    };
    
    /**
     * 控制超时字段更改后调用此函数，函数内容待定
     * @param int $new_web_timeout 新的控制超时设置
     */
    $change_web_timeout = function ($old_web_timeout, $new_web_timeout) {
        // 比较判断控制超时字段值是否更改
        if ((strcmp($old_web_timeout, $new_web_timeout) != 0)) {
            // to do
        }
    };

    /**
     * 检查告警选项之期望保留天数字段是否合法
     * @param int $save_days 期望保留天数
     * @return bool true表示合法，false表示不合法
     */
    $check_save_days = function ($save_days) {
        // $save_days的范围是1~9999
        return ((is_int($save_days)) && ($save_days >= 1)
            && ($save_days <= 9999));
    };

    /**
     * 检查告警选项之审计日志磁盘预警百分比字段是否合法
     * @param int $alarm_percent 审计日志磁盘预警百分比
     * @return bool true表示合法，false表示不合法
     */
    $check_alarm_percent = function ($alarm_percent) {
        // $alarm_percent的范围为30~95
        return ((is_int($alarm_percent)) && ($alarm_percent >= 30)
            && ($alarm_percent <= 95));
    };

    /**
     * 检查自动删除最大保留天数是否合法
     * @param int $del_day 自动删除最大保留天数
     * @return bool true表示合法，false表示不合法
     */
    $check_del_day = function ($del_day) {
        // $del_day的范围是1~365
        return ((is_int($del_day)) && ($del_day >= 1)
            && ($del_day <= 365));
    };
    
    /**
     * 检查创建证书传进来的参数长度是否合法
     * @param int $crt_host host参数
     * @return bool true表示合法，false表示不合法
     */
    $check_crt_host_len = function($crt_host) {
        return !(strlen($crt_host) > 60);
    };

    /**
     * 检查创建证书传进来的参数是否合法
     * @param int $crt_host host参数
     * @return bool true表示合法，false表示不合法
     */
    $check_crt_host_invalid = function($crt_host) {
        return ($crt_host != "127.0.0.1");
    };

    /**
     * 检查创建证书传进来的参数是否合法
     * @param int $crt_host host参数
     * @return bool true表示合法，false表示不合法
     */
    $check_crt = function ($crt_host) use (&$check_crt_host_len, &$check_crt_host_invalid) {
        if (!$check_crt_host_len($crt_host)) {
            return false;
        }

        if (!$check_crt_host_invalid($crt_host)) {
            return false;
        }

        return true;
    };
    
    /**
     * 检查系统选项之发送邮箱字段是否合法
     * @param string $mail_sender 发送邮箱
     * @return bool true表示合法，false表示不合法
     */
    $check_mail_sender = function ($mail_sender) {
        // 不能为空，长度最大100
        if (empty($mail_sender) || (strlen($mail_sender) > 100)) {
            return false;
        }
        // 合符邮箱地址格式
        return (preg_match(
            '/^[_a-zA-Z0-9_\-\.]+@[_a-zA-Z0-9\-]+\.[_a-zA-Z0-9\-\.]+$/',
            $mail_sender) != 0);
    };

    /**
     * 检查系统选项之接收邮箱字段是否合法
     * @param string $mail_receive 接收邮箱
     * @return bool true表示合法，false表示不合法
     */
    $check_mail_receive = function ($mail_receive) use (&$check_mail_sender) {
        // 不能为空
        if (empty($mail_receive)) {
            return false;
        }
        // 分号分割，拆分
        $mail_arr = explode(";", $mail_receive);
        // 最多100个接收邮箱
        if (count($mail_arr) > 100) {
            return false;
        }
        foreach ($mail_arr as $mail) {
            // 每一个邮件地址都必须符合格式
            if (!call_user_func($check_mail_sender, $mail)) {
                return false;
            }
        }
        return true;
    };

    /**
     * 检查系统选项之邮件服务器字段是否合法
     * @param string $mail_serv 邮件服务器地址
     * @return bool true表示合法，false表示不合法
     */
    $check_mail_serv = function ($mail_serv) {
        // 不能为空并且长度<=100
        if (empty($mail_serv) || (strlen($mail_serv) > 100)) {
            return false;
        }
        // 合符邮件服务器格式
        return (preg_match(
            '/^[_a-z0-9\-]+(\.[_a-z0-9\-]+)*$/i', $mail_serv) != 0);
    };

    /**
     * 检查系统选项之用户名字段是否合法
     * @param string $mail_user 系统选项之用户名字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_mail_user = function ($mail_user){
        // 长度<=100，可以为空
        return (strlen($mail_user) <= 100);
    };

    /**
     * 检查系统选项之密码字段是否合法
     * @param string $mail_pwd 检查系统选项之密码字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_mail_pwd = function ($mail_pwd) {
        // 长度<=100，可以为空
        return (strlen(ldb_dectrypt($mail_pwd)) <= 100);
    };
    
    /**
     * 检查端口是否合法
     * @param string $mail_port 检查系统选项之端口字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_mail_port = function ($mail_port) {
        return ($mail_port > 0 && $mail_port <= 65535);
    };

    /**
     * 检查系统选项之邮件服务器配置字段是否合法
     * @param array $smtp_conf 邮件服务器配置字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_smtp_conf = function ($smtp_conf) use (&$check_mail_receive,
        &$check_mail_sender, &$check_mail_serv, &$check_mail_port, &$check_mail_user,
        &$check_mail_pwd) {
        // smtp_conf字段格式，此数组中的非false值为其对应的检测函数名
        $smtp_struct = array(
            "mail_receive"  => "check_mail_receive",
            "mail_sender"   => "check_mail_sender",
            "mail_serv"     => "check_mail_serv",
            "mail_port"     => "check_mail_port",
            "check_user"    => false,
            "mail_user"     => "check_mail_user",
            "mail_pwd"      => "check_mail_pwd"
        );
        foreach ($smtp_conf as $key => $value) {
            // 应不应该有这个字段
            if (!isset($smtp_struct[$key])) {
                return false;
            }
            // 字段值是否合法
            if ($smtp_struct[$key]) {
                if (!call_user_func($$smtp_struct[$key], $value)) {
                    return false;
                }
            }
            // 此字段的值合法
            unset($smtp_struct[$key]);
        }
        // 搜索字段都合法
        if (!empty($smtp_struct)) {
            return false;
        }
        // 验证用户名密码为true时用户名/密码不能为空
        if ($smtp_conf["check_user"] && ((strlen($smtp_conf["mail_user"]) == 0)
            || (strlen($smtp_conf["mail_pwd"]) == 0))) {
                return false;
        }
        return true;
    };

    /**
     * 检查系统选项之控制超时字段是否合法
     * @param int $web_timeout 控制超时字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_web_timeout = function ($web_timeout) {
        // $web_timeout的范围是1~1440
        return ((is_int($web_timeout)) && ($web_timeout >= 1)
            && ($web_timeout <= 1440));
    };

    /**
     * 检查报表生成时间之自动删除N天以前的报表字段是否合法
     * @param int $report_days 自动删除N天以前的报表字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_report_days = function ($report_days) {
        // 内置$report_days的范围是1~90，外置是0~1000
        if (ldb_is_linux()) {
            return ((is_int($report_days)) && ($report_days >= 1)
                && ($report_days <= 90));
        }
        return ((is_int($report_days)) && ($report_days >= 1)
            && ($report_days <= 1000));

    };

    /**
     * 检查报表生成时间之最多保存N份报表字段是否合法
     * @param int $report_nums 最多保存N份报表字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_report_nums = function ($report_nums) {
        // 内置$report_nums的范围是0~10000，外置是0~100000
        if (ldb_is_linux()) {
            return ((is_int($report_nums)) && ($report_nums >= 0)
                && ($report_nums <= 10000));
        }
        return ((is_int($report_nums)) && ($report_nums >= 0)
            && ($report_nums <= 100000));
    };

    /**
     * 检查报表生成时间之最多导出N万条日志字段是否合法
     * @param int $log_max_export 最多导出N万条日志字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_log_max_export = function ($log_max_export) {
        // $log_max_export的范围为1~100
        return ((is_int($log_max_export)) && ($log_max_export >= 1)
            && ($log_max_export <= 100));
    };
    
    /**
     * 检查系统上的实时端口
     * @param int $http_port web端口字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_real_port = function($change_port) {
        $command = "netstat -aon";
        exec($command, $output, $ret);
        if ($ret != 0 ) {
            return false;
        }
        foreach($output as $one) {
            if (strpos($one, "TCP") === false) {
                continue;
            }
            $records = explode(" ", $one);
            $src_ip = $records[6];
            $one_arr = explode(":", $src_ip);
            $src_port = $one_arr[count($one_arr)-1];
            
            if ($src_port == $change_port) {
                return false;
            }
        }
        return true;
    };

    /**
     * 检查高级选项之web端口字段是否合法
     * @param int $http_port web端口字段值
     * @return bool true表示合法，false表示不合法
     */
    $check_http_port = function ($http_port) use (&$check_real_port, &$get_conf_port) {
        // 端口取值范围1~65535
        if (!is_int($http_port) || $http_port < 1 || $http_port > 65535) {
            return false;
        }
        $port_ini = ldb_ext_root()."../dc/config/forbit_ports.ini";
        $lock = $port_ini.".lock";
        $instance = new AutoLock(LOCK_SH | LOCK_NB);
        $init_time = time();
        $success = false;
        // 加锁，五秒超时
        while (time() < $init_time + 5) {
            $success = $instance->lock(array($lock));
            if ($success) {
                break;
            }
            sleep(1);
        }
        if (!$success) {
            return false;
        }
        $ports_data = parse_ini_file($port_ini);
        // 检测端口冲突
        if (in_array($http_port, $ports_data)) {
            return false;
        }
        
        // 如果是当前端口,不检查
        $cur_port = $get_conf_port();
        if ($cur_port == $http_port) {
            return true;
        }
        
        // 检测实时端口
        $ret = $check_real_port($http_port); 
        if ($ret == false) {
            return false;
        }
        
        return true;
    };

    /**
     * 检查$value是否满足$struct格式
     * 并且检查特定字段是否合法（$struct中值不为false的字段）
     * @param array $value 要检查的值
     * @param array $struct 要检查的值需要满足的结构，其中值不为false的字段为其检验函数
     * @return bool true表示合法，false表示不合法
     */
    $check_key_value = function ($value, $struct) use (
        &$check_save_days, &$check_alarm_percent, &$check_smtp_conf,
        &$check_web_timeout, &$check_report_days, &$check_report_nums,
        &$check_log_max_export, &$check_http_port, &$check_del_day,
        &$check_crt) {
        foreach ($value as $key => $key_value) {
            // 该字段是否应该存在
            if (!isset($struct[$key])) {
                return false;
            }
            // 是否应该检查该字段对应的值
            if ($struct[$key]) {
                // 该字段的值不能为空
                // 证书颁与者$conf_struct["advance"]["host"] 允许为空
                if (empty($key_value) && $key != "host") {
                    return false;
                }
                // 检查该字段的值是否合法
                if (!call_user_func($$struct[$key], $key_value)) {
                    return false;
                }
            }
            // 该字段的值合法，删除该字段
            unset($struct[$key]);
        }
        // 是否所有字段的值都合法
        return (empty($struct));
    };

    /**
     * 获取系统配置数据结构，对应值不为false的字段，其值为该字段检测函数名
     * @return array $conf_struct conf_data字段结构的数组
     */
    $get_conf_struct = function () {
         $conf_struct = array(
            "sys"       => array(
                "smtp_conf"     => "check_smtp_conf",
            ),
            "report"    => array(
                "auto_del"          => false,
                "report_days"       => "check_report_days",
                "max_save"          => false,
                "report_nums"       => "check_report_nums"
            ),
            "advance"   => array(
                "show_record_count_yes"   => false,
                "show_record_count_no"    => false,
                "group_flux_only_yes"   => false,
                "group_flux_only_no"    => false,
                "log_max_export"        => "check_log_max_export"
            ),
            "search"    => array(
                "search_open"       => false,
                "search_close"      => false,
                "vt_search_open"      => false,
                "vt_search_close"      => false
            )
        );
        // 外置如下字段不同
        if (!ldb_is_linux()) {
            $conf_struct["sync"] = array(
                "log_dir"       => false,
                "attach_dir"    => false,
                "encode"        => false,
                "unencode"      => false
            );
            $conf_struct["alarm"] = array(
                "del_day_enable"    => false,
                "del_day"           => "check_del_day",
                "save_days"         => "check_save_days",
                "alarm_percent"     => "check_alarm_percent",
                "del_percent"       => false,
                "disk_alarm"        => false,
                "dc_abnormal_alarm" => false,
                "serial_alarm"      => false
            );
            $conf_struct["advance"]["http_port"] = "check_http_port";
            $conf_struct["advance"]["host"] = "check_crt";
            $conf_struct["search"]["at_index_open"] = false;
            $conf_struct["search"]["at_index_close"] = false;
            $conf_struct["sys"]["web_timeout"] = "check_web_timeout";
            $conf_struct["sys"]["binary_1000"] = false;
            $conf_struct["sys"]["binary_1024"] = false;
            $conf_struct["sys"]["flow_unit_byteps"] = false;
            $conf_struct["sys"]["flow_unit_bitps"] = false;
        }
        return $conf_struct;
    };
    
    /**
     * 校验ip合法性
     * @param string $ip ip或域名
     * @return bool 1代表不合法，0代表合法
     */
    function check_invalid_ip($ip) {
        if (empty($ip)) {
            return 0;
        }
        $arr = explode('.',$ip);
        if(count($arr) != 4){
            return 1;
        }else{
            for($i = 0;$i < 4;$i++){
                if(($arr[$i] <'0') || ($arr[$i] > '255')){
                    return 1;
                }
            }
        }
        return 0;
    }

    /**
     * 校验域名合法性
     * @param string $domain ip或域名
     * @return bool 1代表不合法，0代表合法
     */
    function check_invalid_domain($domain) {
        if (empty($domain)) {
            return 0;
        }
        $domain_preg = "/^([a-z\d]([-_a-z\d]{0,61}[a-z\d])?\.)+(?:[a-z]{2,6})$/i";
        $domain_ret_val = preg_match($domain_preg, $domain);
        if ($domain_ret_val === 0) {
            return 1;
        }
        return 0;
    }

    /**
     * 检查系统配置参数
     * @param array $conf_data 系统配置参数
     * @return bool true表示合法，false表示不合法
     */
    $check_conf_data = function ($conf_data) use (&$check_key_value,
        &$get_conf_struct, &$check_invalid_ip, &$check_invalid_domain) {
        // 参数不能为空
        if (empty($conf_data)) {
            return false;
        }
        // 校验域名或ip完整性
        $ip_domain = $conf_data["advance"]["host"];
        $ip_ret_val = check_invalid_ip($ip_domain);
        $domain_ret_val = check_invalid_domain($ip_domain);
        if (($domain_ret_val === 1) && ($ip_ret_val === 1)) {
            return false;
        }
        // 检查结构,定义了conf_data的结构,不为false的值对应为其检测函数名
        $conf_struct = call_user_func($get_conf_struct);
        // 判断结构是否正确，检测子结构
        foreach ($conf_data as $key => $value) {
            // 是不是应有的字段
            if (!isset($conf_struct[$key])) {
                return false;
            }
            // 对应字段值是否为空
            if (empty($value)) {
                return false;
            }
            // 检查key对应的值
            if (!call_user_func($check_key_value, $value,
                $conf_struct[$key])) {
                return false;
            }
            // 检查该key字段无误
            unset($conf_struct[$key]);
        }
        // 是否所有字段无误
        return (empty($conf_struct));
    };

    /**
     * 检查前端传入的操作数据是否合法
     * @param object $object 操作对象
     * @param array $data 操作数据
     * @return bool true表示合法，false表示不合法
     */
    $check_form = function ($object, $data) use (&$check_smtp_conf,
        &$check_conf_data, &$check_http_port, &$check_crt_host_len,
        &$check_crt_host_invalid) {
        // 操作数据不能为空，且操作不能为空
        if (empty($data) || empty($data["opr"])) {
            $object->lasterr = ldb_get_lang("SYS_CONF_OPR_DATA_ERROR");
            return false;
        }
        // 根据不同的操作做检查
        switch ($data["opr"]) {
        case "list":
            return true;
        // 测试邮件有效性操作需要验证邮件服务器配置数据
        case "test_mail":
            if (call_user_func($check_smtp_conf, $data["smtp_conf"])) {
                return true;
            }
            break;
        // 更改系统配置需要验证配置数据是否合法
        case "modify":
            if (call_user_func($check_conf_data, $data["conf_data"])) {
                return true;
            }
            // 操作数据错误时，web端口冲突特殊处理需要提示
            if (!ldb_is_linux() && 
                ((empty($data["conf_data"]["advance"]["http_port"]))
                || !(call_user_func($check_http_port,
                $data["conf_data"]["advance"]["http_port"])))) {
                $object->lasterr = ldb_get_lang("SYS_CONF_PORT_ERROR");
                return false;
            }
            // 证书颁与者名字不能超过60字符
            if (!ldb_is_linux() &&
                ((isset($data["conf_data"]["advance"]["host"]))
                && !(call_user_func($check_crt_host_len,
                $data["conf_data"]["advance"]["host"])))) {
                $object->lasterr = ldb_get_lang("SYS_CONF_CRT_LEN_ERROR");
                return false;
            }
            // 证书不能颁发给127.0.0.1
            if (!ldb_is_linux() &&
                ((isset($data["conf_data"]["advance"]["host"]))
                && !(call_user_func($check_crt_host_invalid,
                $data["conf_data"]["advance"]["host"])))) {
                $object->lasterr = ldb_get_lang("SYS_CONF_CRT_HOST_ERROR");
                return false;
            }
        default:
            break;
        }
        $object->lasterr = ldb_get_lang("SYS_CONF_OPR_DATA_ERROR");
        return false;
    };

    /**
     * 获取配置文件名（路径+名字）
     * @return string 配置文件sysconfig.js所在的路径+名字
     */
    $get_conf_name = function () {
        // 外置
        $file_name = ldb_ext_root().'../dc/config/sys_config.js';
        if (ldb_is_linux()) {
            // 内置
            $file_name = ldb_ext_root().'../config/sys_config.js';
        }
        return $file_name;
    };
    
    /**
     * 读配置文件sysconfig.js并json解码
     * @return array json解码后的文件数据
     */
    $read_sys_config = function () use ($get_conf_name) {
        $file_name = call_user_func($get_conf_name);
        if (!file_exists($file_name)) {
            return NULL;
        }
        $file_data = file_get_contents($file_name);
        if (empty($file_data)) {
            return NULL;
        }
        return json_decode($file_data, true);
    };

    /**
     * 写入到配置文件
     * @param array $conf_data 要写入文件的数据
     * @return int 0表示合法，非0表示不合法
     */
    $write_sys_config = function ($conf_data) use ($get_conf_name) {
        $file_name = call_user_func($get_conf_name);
        $file_data = json_encode($conf_data);
        if (empty($file_data)) {
            return -1;
        }
        // 写入文件
        if (file_put_contents($file_name, $file_data) > 0) {
            return 0;
        }
        return -1;
    };

    /**
     * 同步选项字段转换给前端
     * @param array $sys_data 同步选项字段
     * @return array 发送给前端的同步选项字段
     */
    $get_sync = function ($sys_data) {
        $log_dir = ldb_get_logdata_path();
        if (!ldb_is_linux()) {
            $log_dir = ldb_iconv( "GBK", "UTF-8", $log_dir);
        }
        return array(
            "log_dir"       => $log_dir,
            "attach_dir"    => $sys_data["attach_dir"],
            "encode"        => 
                ($sys_data["encode_attach"] === "on"),
            "unencode"      => 
                ($sys_data["encode_attach"] === "off")
        );
    };

    /**
     * 告警选项字段转换给前端
     * @param array $alarm_data 告警选项字段
     * @return array 发送给前端的告警选项字段
     */
    $get_alarm = function ($alarm_data) {
        $del_day = 365;
        $del_day_enable = false;
        $serial_alarm = true;
        if (array_key_exists("del_day", $alarm_data)) {
            $del_day = $alarm_data["del_day"] ? $alarm_data["del_day"] : 365;
        }
        if (array_key_exists("del_day_enable", $alarm_data)) {
            $del_day_enable = ($alarm_data["del_day_enable"] === "on");
        }
        if (array_key_exists("serial_alarm", $alarm_data)) {
            $serial_alarm = ($alarm_data["serial_alarm"] === "on");
        }
        return array(
            "del_day_enable"    => $del_day_enable,
            "del_day"           => $del_day,
            "save_days"         => $alarm_data["save_days"],
            "alarm_percent"     => $alarm_data["alarm_percent"],
            "del_percent"       => $alarm_data["del_percent"],
            "disk_alarm"        => 
                ($alarm_data["disk_alarm"] === "on"),
            "dc_abnormal_alarm" =>
                ($alarm_data["dc_abnormal_alarm"] === "on"),
            "serial_alarm" =>$serial_alarm,
        );
    };
    
    /**
     * 系统选项字段转换给前端
     * @param array $sys_data 系统选项字段
     * @return array 发送给前端的系统选项字段
     */
    $get_sys = function ($sys_data) use($pwd_const){
        $ret_arr = array();
        // 密码返回常量fixpwd135792468
        $ret_arr["smtp_conf"] = array( // 邮件服务器配置字段
                                "mail_receive"  => $sys_data["smtp_conf"]["mail_receive"],
                                "mail_sender"   => $sys_data["smtp_conf"]["mail_sender"],
                                "mail_serv"     => $sys_data["smtp_conf"]["mail_serv"],
                                "mail_port"     => 
                                    (isset($sys_data["smtp_conf"]["mail_port"]) 
                                    ? $sys_data["smtp_conf"]["mail_port"] : 25),
                                "check_user"    => 
                                    (($sys_data["smtp_conf"]["check_user"] === "on")
                                    ? true : false),
                                "mail_user"     => $sys_data["smtp_conf"]["mail_user"],
                                "mail_pwd"      => ldb_enctrypt($pwd_const)
                            );
        // 内置没有控制台超时，依赖于网关的超时设置
        if (!ldb_is_linux()) {
           $ret_arr["web_timeout"] = $sys_data["web_timeout"]; 
           if (!isset($sys_data["binary"])) {
                $ret_arr["binary_1000"] = true;
               $ret_arr["binary_1024"] = false;
           } else {
               $ret_arr["binary_1000"] = ($sys_data["binary"] == 1000);
               $ret_arr["binary_1024"] = ($sys_data["binary"] == 1024);
           }

           $ret_arr["flow_unit_byteps"] = ($sys_data["flow_unit"] === "byteps");
           $ret_arr["flow_unit_bitps"] =  ($sys_data["flow_unit"] === "bitps");
        }
        return $ret_arr;
    };

    /**
     * 报表生成时间字段转换给前端
     * @param array $report_data 报表生成时间字段
     * @return array 发送给前端的报表生成时间字段
     */
    $get_report = function ($report_data) {
        return array(
            "auto_del"          => 
                ($report_data["report_del_mode"] === "day"),
            "report_days"       => $report_data["report_days"],
            "max_save"          => 
                ($report_data["report_del_mode"] === "num"),
            "report_nums"       => $report_data["report_nums"]
        );
    };

    /**
     * 高级选项字段转换给前端
     * @param array $adv_data 高级选项字段
     * @return array 发送给前端的高级选项字段
     */
    $get_advance = function ($adv_data) {
        $ret = array(
            "show_record_count_yes" => ($adv_data["show_record_count"] === "on"),
            "show_record_count_no"  => ($adv_data["show_record_count"] === "off"),
            "group_flux_only_yes"   => ($adv_data["group_flux_only"] === "on"),
            "group_flux_only_no"   => ($adv_data["group_flux_only"] === "off"),
            "log_max_export"    => $adv_data["log_max_export"]
        );
        // 外置的话此字段多了一个web端口
        if (!ldb_is_linux()) {
            $ret["http_port"] = $adv_data["http_port"];
            $crt_host = "";
            if (array_key_exists("host", $adv_data)) {
                $crt_host = $adv_data["host"];
            }
            $ret["host"] = $crt_host;
        }
        return $ret;
    };

    /**
     * 搜索选项字段转换给前端
     * @param array $search_data 搜索选项字段
     * @return array 发送给前端的搜索选项字段
     */
    $get_search = function ($search_data) {
        $search_data["vt_search_center"] = 
            isset($search_data["vt_search_center"]) ? $search_data["vt_search_center"] : "off";
        $ret = array(
            "search_open"       => 
                ($search_data["search_center"] === "on"),
            "search_close"      => 
                ($search_data["search_center"] === "off"),
            "vt_search_open"       => 
                ($search_data["vt_search_center"] === "on"),
            "vt_search_close"      => 
                ($search_data["vt_search_center"] === "off")
        );
        
        // 外置开启搜索中心和开启附件内容索引有关联
        if (!ldb_is_linux()) {
            // 搜索中心开启附件内容索引才能开启
            if ($search_data["search_center"] === "off") {
                $search_data["attach_index"] = "off";
            }
            $ret["at_index_open"] = 
                ($search_data["attach_index"] === "on");
            $ret["at_index_close"] = 
                $search_data["attach_index"] === "off";
        }
        return $ret;
    };

    /**
     * 更改同步选项设置
     * @param array $sync_conf 原同步选项设置
     * @param array $sync_data 更改值
     * @return array $sync_conf 更改后的同步选项字段值
     */
    $set_sync = function ($sync_conf, $sync_data) {
        $sync_conf["encode_attach"] = ($sync_data["encode"] ? "on" : "off");
        return $sync_conf;
    };

    /**
     * 更改告警选项设置
     * @param array $alarm_conf 原告警选项设置
     * @param array $alarm_data 更改值
     * @return array $alarm_conf 更改后的告警选项字段值
     */
    $set_alarm = function ($alarm_conf, $alarm_data) {
        $alarm_conf["del_day_enable"] = $alarm_data["del_day_enable"] ? "on" : "off";
        $alarm_conf["del_day"] = $alarm_data["del_day"];
        $alarm_conf["save_days"] = $alarm_data["save_days"];
        $alarm_conf["alarm_percent"] = $alarm_data["alarm_percent"];
        // 磁盘预警、系统异常告警 true->on  false->off
        $alarm_conf["disk_alarm"] = $alarm_data["disk_alarm"] ? "on" : "off";
        $alarm_conf["dc_abnormal_alarm"] = $alarm_data["dc_abnormal_alarm"]
            ? "on" : "off";
        $alarm_conf["serial_alarm"] = $alarm_data["serial_alarm"] ? "on" : "off";
        return $alarm_conf;
    };

    /**
     * 更改系统选项设置
     * @param array $sys_conf 原系统选项设置
     * @param array $sys_data 更改值
     * @return array $sys_conf 更改后的系统选项字段值
     */
    $set_sys = function ($sys_conf, $sys_data) use($pwd_const){
        // 邮件服务器配置
        $smtp_conf = $sys_data["smtp_conf"];
        $sys_conf["smtp_conf"]["mail_receive"] = $smtp_conf["mail_receive"];
        $sys_conf["smtp_conf"]["mail_sender"] = $smtp_conf["mail_sender"];
        $sys_conf["smtp_conf"]["mail_serv"] = $smtp_conf["mail_serv"];
        $sys_conf["smtp_conf"]["mail_port"] = $smtp_conf["mail_port"];
        $sys_conf["smtp_conf"]["check_user"] = $smtp_conf["check_user"]
            ? "on" : "off";
        $sys_conf["smtp_conf"]["mail_user"] = $smtp_conf["mail_user"];
        // 相等的话，说明没有改变密码
        if ($pwd_const != ldb_dectrypt($smtp_conf["mail_pwd"])) {
            $sys_conf["smtp_conf"]["mail_pwd"] = $smtp_conf["mail_pwd"];
        }
        // 内置数据中心没有控制台超时
        if (!ldb_is_linux()) {
            $sys_conf["web_timeout"] = $sys_data["web_timeout"];
            $sys_conf["binary"] = $sys_data["binary_1000"] ? 1000 : 1024;
            $sys_conf["flow_unit"] =  $sys_data["flow_unit_byteps"]
            ? "byteps" : "bitps";
        }
        return $sys_conf;
    };

    /**
     * 更改报表生成时间选项设置
     * @param array $report_conf 原报表生成时间选项设置
     * @param array $report_data 更改值
     * @return array $report_conf 更改后的报表生成时间选项字段值
     */
    $set_report = function ($report_conf, $report_data) {
        $report_conf["report_del_mode"] = $report_data["auto_del"]
            ? "day" : "num";
        $report_conf["report_days"] = $report_data["report_days"];
        $report_conf["report_nums"] = $report_data["report_nums"];
        return $report_conf;
    };

    /**
     * 更改高级选项设置
     * @param array $advance_conf 原高级选项设置
     * @param array $advance_data 更改值
     * @return array $advance_conf 更改后的高级选项字段值
     */
    $set_advance = function ($advance_conf, $advance_data) {
        $advance_conf["show_record_count"] = 
            ($advance_data["show_record_count_yes"] ? "on" : "off");
        $advance_conf["group_flux_only"] = 
            ($advance_data["group_flux_only_yes"] ? "on" : "off");
        $advance_conf["log_max_export"] = $advance_data["log_max_export"];
        // 外置要考虑web端口
        if (!ldb_is_linux()) {
            $advance_conf["http_port"] = $advance_data["http_port"];
            $advance_conf["host"] = $advance_data["host"];
        }
        return $advance_conf;
    };

    /**
     * 更改搜索选项设置
     * @param array $search_conf 原搜索选项设置
     * @param array $search_data 更改值
     * @return array $search_conf 更改后的搜索选项字段值
     */
    $set_search = function ($search_conf, $search_data) {
        $search_conf["search_center"] = $search_data["search_open"]
            ? "on" : "off";
        $search_conf["vt_search_center"] = $search_data["vt_search_open"]
            ? "on" : "off";
        if (!ldb_is_linux()) {
            $search_conf["attach_index"] = $search_data["at_index_open"]
                ? "on" : "off";
            // search_center和attach_index字段关联
            if ($search_data["search_open"] == false) {
                $search_conf["attach_index"] = "off";
            }
        }
        return $search_conf;
    };

    /**
     * 制作crt的准备工作
     * @param string $config_home apache配置目录
     * @param string $ca_home 制作crt的目录
     * @param string $comm_name 证书颁与者名
     * @param string $s_key 记录秘钥的文件
     * @param string $s_csr 证书请求文件
     * @param string $s_crt 经根证书签名的证书文件
     * @param string $ext_file 签名配置文件--chrome需要此文件，否则签名的证书在chrome中仍无效
     * @param string $s_key_backup 记录秘钥的文件的备份
     * @param string $s_crt_backup 经根证书签名的证书文件的备份
     * @return 0清理成功，-1清理失败
     */
    $pre_make = function ($config_home, $ca_home, $comm_name, 
                          $s_key, $s_csr, $s_crt, $ext_file) {
		$s_key_backup = $config_home.$s_key."_backup";
        $s_crt_backup = $config_home.$s_crt."_backup";
        if (is_file($s_key_backup)) {
            unlink($s_key_backup);
        }
        if (is_file($s_crt_backup)) {
            unlink($s_crt_backup);
        }
        if (is_file($ca_home.$s_key)) {
            unlink($ca_home.$s_key);
        }
        if (is_file($ca_home.$s_csr)) {
            unlink($ca_home.$s_csr);
        }
        if (is_file($ca_home.$s_crt)) {
            unlink($ca_home.$s_crt);
        }
        if (is_file($config_home.$s_key)) {
            if (!rename($config_home.$s_key, $s_key_backup)) {
                ldb_error("pre_make copy $config_home.$s_key to $s_key_backup fail.");
                return -1;
            }
        }
        if (is_file($config_home.$s_crt)) {
            if (!rename($config_home.$s_crt, $s_crt_backup)) {
                ldb_error("pre_make copy $config_home.$s_crt to $s_crt_backup fail.");
                rename($s_key_backup, $config_home.$s_key);
                return -1;
            }
        }
        $info = "keyUsage = nonRepudiation, digitalSignature, keyEncipherment".PHP_EOL;
        $info .= "subjectAltName=@SubjectAlternativeName".PHP_EOL;
        $info .= "[ SubjectAlternativeName ]".PHP_EOL;
        if (!filter_var($comm_name, FILTER_VALIDATE_IP)) {
            $info .= "DNS.1 = ".$comm_name.PHP_EOL;
        } else {
            $info .= "IP.1 = ".$comm_name.PHP_EOL;
        }
        if (file_put_contents($ca_home.$ext_file, $info) == false) {
            ldb_error("make crt file_put_contents to $ca_home$ext_file fail.");
            return -1;
        }
        return 0;
    };
    
    /**
     * 制作crt
     * @param array $comm_name 证书颁与者名
     * @return 0制作成功，-1制作失败
     */
    $make_crt = function ($comm_name) use (&$pre_make) {
        $openssl_exe = ldb_ext_root() ."/../apache/bin/openssl.exe";
        $ca = "cacert.pem";
        $ca_key = "cakey.pem";
        $s_key = "server.key";
        $s_csr = "server.csr";
        $s_crt = "server.crt";
        $s_key_origin = "server_origin.key";
        $s_crt_origin = "server_origin.crt";
        $ext_file = "extfile.txt";
        $config_home = ldb_ext_root()."/../apache/conf/";
        $ca_home = $config_home. "/signCA/";
		$config_homes = array(ldb_ext_root()."/../apache/conf/",
							  ldb_ext_root()."/../nginx/conf/");
        if (!is_dir($ca_home) 
            || !is_file($ca_home.$ca) 
            || !is_file($ca_home.$ca_key)) {
            ldb_error("$ca_home or $ca_home.$ca or $ca_home.$ca_key doesn't exist. make crt fail.");
            return -1;
        }
		foreach($config_homes as $config_home){
			if ($pre_make($config_home, $ca_home, $comm_name, $s_key, $s_csr,
                     $s_crt, $ext_file) != 0) {
				ldb_error("pre_make fail.");
				return -1;
			}
        
			// 如果证书颁与者为空，则将服务器证书恢复成默认证书
			if ($comm_name == "") {
				if (!is_file($ca_home.$s_key_origin)
					|| !is_file($ca_home.$s_crt_origin)) {
					ldb_error("origin crt or key doesn't exist.");
					return -1;
				}
				if (!copy($ca_home.$s_key_origin, $config_home.$s_key)) {
					ldb_error("copy $ca_home/$s_key_origin to $config_home/$s_key fail.");
					rename($config_home.$s_key."_backup", $config_home.$s_key);
					rename($config_home.$s_crt."_backup", $config_home.$s_crt);
					return -1;
				}
				if (!copy($ca_home.$s_crt_origin, $config_home.$s_crt)) {
					ldb_error("copy $ca_home/$s_crt_origin to $config_home/$s_crt fail.");
					unlink($config_home.$s_key);
					rename($config_home.$s_key."_backup", $config_home.$s_key);
					rename($config_home.$s_crt."_backup", $config_home.$s_crt);
					return -1;
				}
				return 0;
			}
		}
        
        $cmd = sprintf("%s genrsa -out %s 2048", "\"$openssl_exe\"", escapeshellarg($ca_home."/". $s_key));
        ldb_exec($cmd, $out, $ret);
        if ($ret != 0) {
            ldb_error("exec $cmd fail.");
            return -1;
        }
        $cmd = sprintf("%s req -new -sha256 -out %s -key %s -config %s -subj %s", 
                        "\"$openssl_exe\"", escapeshellarg($ca_home."/". $s_csr), escapeshellarg($ca_home."/". $s_key),
                        escapeshellarg($ca_home."/openssl.cnf"), escapeshellarg("/CN=".$comm_name));
        ldb_exec($cmd, $out, $ret);
        if ($ret != 0) {
            ldb_error("exec $cmd fail.");
            return -1;
        }
        $cmd = sprintf("%s x509 -sha256 -req -in %s -CA %s -CAkey %s -CAcreateserial -out %s -startdate -days 5000 -passin pass:1234 -extfile %s", 
                        "\"$openssl_exe\"", escapeshellarg($ca_home."/". $s_csr), escapeshellarg($ca_home."/". $ca),
                        escapeshellarg($ca_home."/". $ca_key), escapeshellarg($ca_home."/". $s_crt), escapeshellarg($ca_home."/extfile.txt"));
        ldb_exec($cmd, $out, $ret);
        if ($ret != 0) {
            ldb_error("exec $cmd fail.");
            return -1;
        }
		foreach($config_homes as $config_home){
			ldb_info("$config_home");
			if (is_file($ca_home.$s_crt)) {
				if (!copy($ca_home.$s_crt, $config_home.$s_crt)) {
					ldb_error("copy $ca_home/$s_crt to $config_home/$s_crt fail.");
					rename($config_home.$s_key."_backup", $config_home.$s_key);
					rename($config_home.$s_crt."_backup", $config_home.$s_crt);
					return -1;
				}
				if (!copy($ca_home.$s_key, $config_home.$s_key)) {
					ldb_error("copy $ca_home/$s_key to $config_home/$s_key fail.");
					unlink($config_home.$s_crt);
					rename($config_home.$s_key."_backup", $config_home.$s_key);
					rename($config_home.$s_crt."_backup", $config_home.$s_crt);
					return -1;
				}
			}
		}
        return 0;
    };
    
    /**
     * 重新生成crt
     * @param array $advance_sys 本地的advance配置数据
     * @param array $advance_conf 前端传入的advance配置数据
     * @return 0重新生成crt成功，1无需重新生成crt，-1生成crt失败
     */
    $rebuild_crt = function ($advance_sys, $advance_conf) use (&$make_crt) {
        if (ldb_is_linux()) {
            return 1;
        }
        if (!is_array($advance_conf)
            || !isset($advance_conf["host"])
            || strlen($advance_conf["host"]) > 60
            || ($advance_conf["host"] == "127.0.0.1")) {
            ldb_warn("rebuild_crt function para invalid.");
            return -1;
        }

        $local_current = $advance_sys["host"];
        $conf_host = $advance_conf["host"];
        if ($local_current != $conf_host) {
            return $make_crt($conf_host);
        }
        
        return 1;
    };

    /**
     * 获取系统配置参数-存储于配置文件sysconfig.js
     * @param object $object 操作对象
     * @param array $data 操作数据
     * @return int 0表示操作成功，非0表示操作失败
     */
    $list = function ($object, $data) use (&$read_sys_config,
                                            &$get_sync,
                                            &$get_alarm,
                                            &$get_sys,
                                            &$get_report,
                                            &$get_advance,
                                            &$get_search) {
        // 读取配置文件
        $config = call_user_func($read_sys_config);
        if (empty($config)) {
            $object->lasterr = ldb_get_lang("SYS_CONF_LIST_FAIL");
            return -1;
        }
        // 构造给前端的结果 内置
        $conf_data = array(
            "sys"       => call_user_func($get_sys, $config["sys"]),
            "report"    => call_user_func($get_report, $config["report"]),
            "advance"   => call_user_func($get_advance, $config["advance"]),
            "search"    => call_user_func($get_search, $config["search"])
        );
        if (!ldb_is_linux()) { // 外置才有的字段赋值
            $conf_data["sync"] = call_user_func($get_sync, $config["sync"]);
            $conf_data["alarm"] = call_user_func($get_alarm, $config["alarm"]);
        }
        ldb_out_data($conf_data);
        return 0;
    };
    
    /**
     * 获取当前配置端口
     * @return mixed 从配置文件读取端口,失败返回-1;
     */
    $get_conf_port = function() use (&$read_sys_config) {
        
        // 解析配置文件
        $sys_config = call_user_func($read_sys_config);
        if ($sys_config == null) {
            return -1;
        }
        
        // 配置文件直接取
        $ret = $sys_config["advance"]["http_port"];
        return $ret;
    };
    
    /**
     * 获取邮件密码
     * @return mixed 从配置文件返回密码字符串,失败返回-1
     */
    $get_email_pass = function() use (&$read_sys_config) {
        
        // 解析配置文件
        $sys_config = call_user_func($read_sys_config);
        if ($sys_config == null) {
            return -1;
        }
        
        // 配置文件直接取
        $conf_pass = $sys_config["sys"]["smtp_conf"]["mail_pwd"];
        $ret = ldb_dectrypt($conf_pass);
        return $ret;
    };

    /**
     * 测试邮件有效性
     * @param object $object 操作对象
     * @param array $data 操作数据
     * @return int 0表示操作成功，非0表示操作失败
     */
    $test_mail = function ($object, $data) use (&$pwd_const, &$get_email_pass) {
        
        // 错误提示
        $object->lasterr = ldb_get_lang("SYS_CONF_TEST_MAIL_FAIL");
        
        // 邮件数据
        $smtp_conf = $data["smtp_conf"];
       
        // 拼装邮件
        $mail = new PHPMailer();
        $mail->IsSMTP();
        $mail->Host = $smtp_conf["mail_serv"];
        //$mail->Port = 25;
        $mail->Port = $smtp_conf["mail_port"];
        
        // 是否启用密码
        if($smtp_conf["check_user"] == "1") {
            $mail->SMTPAuth = true;
            $mail->Username = $smtp_conf["mail_user"];
            
            // 如果密码是默认密码,从配置文件读取
            $email_pass = $smtp_conf["mail_pwd"];
            if (ldb_dectrypt($email_pass) == $pwd_const) {
                $ret = $get_email_pass();
                if ($ret == -1) {
                    return -1;
                }
                $mail->Password = $ret;
            } else {
                $mail->Password = ldb_dectrypt($email_pass);
            }
        } else {
            $mail->SMTPAuth = false;
        }

        // 发件人地址
        $mail->From = $smtp_conf["mail_sender"];
        
        // 收件人地址
        $addresses = explode(";", $smtp_conf["mail_receive"]);
        foreach ($addresses as $val) {
            if (false !== strpos($val, '@')) {
                $mail->AddAddress($val);
            }
        }
        
        // set word wrap to 50 characters
        $mail->WordWrap = 50;
        
        // set email format to HTML
        $mail->IsHTML(true);
        if (ldb_is_linux()) {
            $content = ldb_get_lang("SYS_CONF_IN_TEST_EMAIL");
        } else {
            $content = ldb_get_lang("SYS_CONF_OUT_TEST_EMAIL");
        }
        $content_time = date("Y-m-d H:i:s");
        $mail->Subject = $content;
        $mail->Body = $content.$content_time;
        $mail->AltBody = $mail->Body;
        
        // 发送邮件
        if(!$mail->Send()) {
            $mail->Host = gethostbyname($smtp_conf["mail_serv"]);
            if (!$mail->Send()) {
                ldb_error("send test mail failed, error is " . $mail->ErrorInfo);
                return -1;
            }
        }
        
        // 成功提示
        $suc_msg = ldb_get_lang("SYS_CONF_TEST_MAIL_SUCCESS");
        ldb_out_msg($suc_msg, NULL);
        
        return 0;
    };

    /**
     * 更改配置数据
     * @param array $config 从文件读出的配置数据，不合法的字段已改为默认值
     * @param array $conf_data 前端传入的配置数据
     * @return array $config 更改后的结果
     */
    $set_conf_data = function ($config, $conf_data) use (&$set_sync,
        &$set_alarm, &$set_sys, &$set_report, &$set_advance, &$set_search) {
        // 保存前端传入的值
        foreach ($conf_data as $key => $value) {
            $func = "set_".$key;
            $config[$key] = call_user_func($$func, $config[$key], $conf_data[$key]);
        }
        return $config;
    };

    /**
     * 更改系统配置
     * @param object $object 操作对象
     * @param array $data 操作数据
     * @return int 0表示操作成功，非0表示操作失败
     */
    $modify = function ($object, $data) use (&$read_sys_config,
        &$write_sys_config, &$change_http_port, &$set_conf_data,
        &$change_web_timeout, &$rebuild_crt) {
        $sys_config = call_user_func($read_sys_config);
        if (empty($sys_config)) { // 读取配置文件失败
            $object->lasterr = ldb_get_lang("SYS_CONF_MODIFY_FAIL");
            return -1;
        }

        $conf_data = $data["conf_data"];
        $change_crt_ret = 1;    //0重新生成crt成功，1无需重新生成crt，-1生成crt失败
        if (!ldb_is_linux()) {
            $change_crt_ret = call_user_func($rebuild_crt, 
                                             $sys_config["advance"],
                                             $conf_data["advance"]);
            if ($change_crt_ret == -1) {
                $object->lasterr = ldb_get_lang("SYS_CONF_MODIFY_FAIL");
                return -1;
            }
        }
        // 更改配置数据
        $conf = call_user_func($set_conf_data, $sys_config, $conf_data);
        if (call_user_func($write_sys_config, $conf) != 0) {
            $object->lasterr = ldb_get_lang("SYS_CONF_MODIFY_FAIL");
            return -1;
        }
        
        $change_port_ret = 0;
        
        if (!ldb_is_linux()) { // http_port更改操作函数
            $change_port_ret = call_user_func($change_http_port, $sys_config["advance"]["http_port"],
                $conf["advance"]["http_port"]);
                    // web_timeout更改操作函数
            call_user_func($change_web_timeout, $sys_config["sys"]["web_timeout"],
                $conf["sys"]["web_timeout"]);
        }
        $msg = ldb_get_lang("SYS_CONF_MODIFY_SUCCESS");
        ldb_out_msg($msg, NULL);
        ldb_mapreduce_invoke("call_method",
                            "util.common.oprlog", 
                            "write_oprlog",
                            ldb_get_lang("SYS_PAGE_SYSCONF_NAME"),
                            ldb_get_lang("COMMON_MODIFY"),
                            $msg);
        
        // 重启apache和nginx
        if (!ldb_is_linux() && ($change_port_ret == 0 || $change_crt_ret == 0)) {
            // 打上重启apache和nginx的标记,由time进程负责重启
            $reboot_apche_flag = ldb_ext_root()."../dc/need_restart_apache";
            $ret = touch($reboot_apche_flag);
            if ($ret == false) {
                // 失败重试一次,如果再失败,不处理
                touch($reboot_apche_flag);
            }
        }
        return 0;
    };
    
    /**
     * 应用入口函数
     * @return int 返回0表示成功，非0表示有错误
     */
    $main = function () use (&$check_form, &$list, &$test_mail, &$modify) {
        // 权限检测
        $result = ldb_mapreduce_invoke("call_method",
                                        "util.common.auth",
                                        "app_auth_check",
                                        "app.web.system.sys_config");
        // 提示无权限
        if ($result === false) {
            ldb_output_arr(array("success"=> false,
                                "msg" => ldb_get_lang("SYS_NO_APP_AUTH"))
                            );
            return 0;
        }
        list($object, $args) = func_get_args();
        $ret = call_user_func($check_form, $object, $args);
        // 操作数据错误
        if (!$ret) {
            return -1;
        }
        
        $lock = "";
        if (ldb_is_linux()) {
            $lock = ldb_ext_root()."../config/sys_config.js.lock";
        } else {
            $lock = ldb_ext_root()."../dc/config/sys_config.js.lock";
        }
        $instance = new AutoLock(LOCK_EX | LOCK_NB);
        $init_time = time();
        $success = false;
        // 加锁，五秒超时
        while (time() < $init_time + 5) {
            $success = $instance->lock(array($lock));
            if ($success) {
                break;
            }
            sleep(1);
        } // 加锁失败
        if (!$success) {
            $object->lasterr = ldb_get_lang("SYS_CONF_FILE_USING");
            return -1;
        }
        return call_user_func($$args["opr"], $object, $args);
    };
    
    /**
     * 返回最后的错误信息
     * @return string 返回最后的错误信息
     */
    $error = function () {
        list($object) = func_get_args();
        return $object->lasterr;
    };
    
    /**
     * 销毁函数
     * @return 
     */
    $destroy = function () {
        list($object) = func_get_args();
        if (!is_null($object)) {
        }
    };
    
    /**
     * 构造应用
     * @return 返回应用对象
     */
    $constructor = function ()
        use (&$main, &$error, &$destroy) {
        return (object)array(
            "main"      => $main,
            "error"     => $error,
            "destroy"   => $destroy,
            "lasterr"   => null,
            "mapreduce" => null
        );
    };
    
    /**
     * 注册应用
     */
    ldb_mapreduce_invoke("register", 
        ldb_mapreduce_invoke("uniqid", __FILE__), $constructor);
});
?>