<?php
/*
 *  本脚本用于安装老版本外置数据中心在线升级机制程序
 */
function getDcRoot() {
    try { 
        $shell   = new COM("WScript.Shell") or die("load WScript.Shell failed!");
        $root    = $shell->RegRead('HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\DataCenter\ImagePath');
        // root: C:\Program Files\Sangfor\DataCenter\dc\DataCenter.exe
        // dst_dir: C:\Program Files\Sangfor\DataCenter
        $dst_dir = dirname(dirname(trim($root, "\"")));
        return $dst_dir;
    } catch (Exception $e) {
        echo "Please install in an Sangfor DataCenter Platform\n";
        echo "error:getDcRoot failed\n";
        exit(1);
    }
}

function getServiceCount($output, $service) {
    if (!is_array($output)) {
        return false;
    }
    $count = 0;
    foreach ($output as $out) {
        if (in_array(trim($out), $service)) {
            $count++;
        }
    }
    return $count;
}

function ldb_rrmdir($dir) { 
    // 符号链接要先删除符号链接，再删除真实路径
    if (is_link($dir)) {
        $real_dir = realpath($dir);
        return unlink($dir) && ldb_rrmdir($real_dir);
    }

    // 如果是文件直接删除
    if (is_file($dir)) {
        return unlink($dir);
    }

    // 如果不是文件不是符号链接也不是目录，直接返回
    if (!is_dir($dir)) {
        return true;
    }   

    // 排除.和..
    $files = array_diff(scandir($dir), array('.', '..')); 
    foreach ($files as $file) { 
        if (!ldb_rrmdir("$dir/$file")) {
            return false;
        }
    }
    return rmdir($dir);
}

$install_service   = array("Sangfor Data Center");
$uninstall_service = array("Sangfor Data Center");

// SP包名称
$SP_NAME = "SP_DC_UPDSSL_01";

/**
 * 更新版本号
 * @param string  $new_version  输出信息
 * @return bool  成功返回true， 失败返回false
 */
function update_version($new_version) {
    $dst_dir = getDcRoot();
    $file_version = getDcRoot()."\\version\\appversion";
    if(!file_exists($file_version)) {
        die("$file_version not exists");
    }
    global $SP_NAME;
    $bak_version = $file_version.".$SP_NAME";
    if (!copy($file_version, $bak_version)) {
        echo "update version, copy $file_version to $bak_version fail \n";
        return false;
    }
    $fp = fopen($file_version, "a+");
    if (!$fp) {
        die("($file_version) open fail");
    }
    $update_time = "update:".date("Y-m-d H:i:s", time());
    fwrite($fp, "$new_version $update_time");
    fwrite($fp, "\r\n");
    fclose($fp);
    return true;
}


function getVersionBuildTime(){
    $file = getDcRoot()."\\version\\appversion";
    $conf = file_get_contents($file);
    $conf_data = preg_split("/[\s,]+/", $conf);
    foreach ($conf_data as $key => $value) {
        if (preg_match("/Build/", $value) != 0) {
            $time = preg_replace('/\D/','',$value);
            return $time;
        }
    }
    return "99999999";
}

function check_nginx() {
	if (file_exists(getDcRoot()."\\nginx")){
        ldb_exec("taskkill /im ba_nginx.exe /f /t 1>nul 2>&1", $output, $ret);
		ldb_rrmdir(getDcRoot()."\\nginx");
	}
	return false;
}

/**
 * 执行外部程序
 * @param string $command 执行命令
 * @param array  $output  输出信息
 * @param int    $ret     返回值
 * @return string 返回执行结果
 */
function ldb_exec($command, &$output, &$ret) {
    $data = exec($command, $output, $ret);
    return $data;
}

function stopService($dst_dir) {
    global $uninstall_service;
    ldb_exec("$dst_dir/apache/bin/stop.bat", $output, $ret);
    foreach($uninstall_service as $server) {
        ldb_exec("net stop \"$server\"",$output, $ret);
    }
    ldb_exec("net start", $output, $ret);
    $count = getServiceCount($output, $uninstall_service);
    if ($count !== 0) {
        echo "stop service failed!\n";
        exit(1);
    }
}

function startService($dst_dir) {
    global $install_service;
	ldb_exec("$dst_dir/apache/bin/start.bat", $output, $ret);
    foreach($install_service as $server) {
        ldb_exec("net start \"$server\"",$output, $ret);
    }
    ldb_exec("net start", $output, $ret);
    $count = getServiceCount($output, $install_service);
    if ($ret || $count !== count($install_service)) {
        echo "check service failed!\n";
        exit(1);
    }
}
function renew_nginx_conf($port) {
    $file_name = getDcRoot()."\\nginx\\conf\\nginx.conf";
    $conf = file_get_contents($file_name);
    $conf_data = explode("\n", $conf);
    foreach ($conf_data as $key => $value) {
        if (preg_match("/^[\s]+listen/", $value) != 0) {
            $conf_data[$key] = "        listen ".$port." ssl;";
            break;
        }
    }
    $file_data = "";
    foreach ($conf_data as $line) {
        $file_data .= $line."\n";
    }
    $tmp_name = getDcRoot()."/httpd-ssl_tmp.js"; // 临时文件
    if (!file_put_contents($tmp_name, $file_data)
        || !rename($tmp_name, $file_name)) {
        unlink($tmp_name);
        return 1;
    }
    return 0;
}

function modify_apache($new_http_port){
    $file_name = getDcRoot()."\\apache\\conf\\extra\\httpd-ssl.conf";
    if (!copy($file_name, $file_name.".bak")) {
        echo "modify_apache, backup $file_name fail \n";
        return 1;
    }
    $conf = file_get_contents($file_name);
    $conf_data = explode("\r\n", $conf);
    foreach ($conf_data as $key => $value) {
        if (preg_match("/^Listen/", $value) != 0) {
            $conf_data[$key] = "Listen $new_http_port";
            continue;
        }
        if (preg_match("/^<VirtualHost _default_:/", $value) != 0) {
            $conf_data[$key] = "<VirtualHost 127.0.0.1:".$new_http_port.">";
            continue;
        }
        if (preg_match("/^ServerName www.example.com:/", $value) != 0) {
            $conf_data[$key] = "ServerName www.example.com:".$new_http_port;
            continue;
        }
    }
    $file_data = "";
    foreach ($conf_data as $line) {
        $file_data .= $line."\r\n";
    }
    $tmp_name = getDcRoot()."/httpd-ssl_tmp.js"; // 临时文件
    if (!file_put_contents($tmp_name, $file_data)
        || !rename($tmp_name, $file_name)) {
        echo "modify_apache, save $file_name fail \n";
        unlink($tmp_name);
        return 1;
    }
    return 0;
}

function modify_php_conf(){
    $file_name = array(getDcRoot()."\\ldb\\php\\php.ini",
                       getDcRoot()."\\ldb\\php\\php-cgi-fcgi.ini"
        );
    foreach($file_name as $file){
        if (!copy($file, $file.".bak")) {
            echo "modify_apache, backup $file fail \n";
            return 1;
        }
        $conf = file_get_contents($file);
        $conf_data = explode("\r\n", $conf);
        foreach ($conf_data as $key => $value) {
            if (preg_match("/^max_input_time =/", $value) != 0) {
                $conf_data[$key] = "max_input_time = 1200";
                continue;
            }
        }
        $file_data = "";
        foreach ($conf_data as $line) {
            $file_data .= $line."\r\n";
        }
        $tmp_name = getDcRoot()."/php_tmp.ini"; // 临时文件
        if (!file_put_contents($tmp_name, $file_data)
            || !rename($tmp_name, $file)) {
            echo "modify_apache, save $file fail \n";
            unlink($tmp_name);
            return 1;
        }
    }
}

function modify_sys_config(){
    $buildTime = getVersionBuildTime();
    if ($buildTime >= "20190815" && $buildTime != "99999999"){
        return 1;
    }
    $patch = array("    \$change_http_port = function (\$old_http_port, \$new_http_port) {",
                    "        if ((strcmp(\$old_http_port, \$new_http_port) == 0)) {",
                    "            return -1;",
                    "        }",
"                    \$file_name = ldb_ext_root().\"../nginx/conf/nginx.conf\";",
"                    \$file_data = file_get_contents(\$file_name);",
"                    \$conf_data = explode(\"\n\", \$file_data);",
"                    foreach (\$conf_data as \$key => \$value) {",
"                        if (preg_match(\"/^[\s]+listen/\", \$value) != 0) {",
"                            \$conf_data[\$key] = \"        listen \".\$new_http_port.\" ssl;\";",
"                            continue;",
"                        }",
"                    }",
"                    \$file_data = \"\";",
"                    foreach (\$conf_data as \$line) {",
"                        \$file_data .= \$line.\"\r\n\";",
"                    }",
"                    \$tmp_name = ldb_get_temp_dir().\"/httpd-ssl_tmp.js\";",
"                    if (!file_put_contents(\$tmp_name, \$file_data)",
"                        || !rename(\$tmp_name, \$file_name)) {",
"                        unlink(\$tmp_name);",
"                        return -1;",
"                    }",
"                    return 0;"
                );
    $file_name = getDcRoot()."\\ldb\\bin\\mapreduce\\app\\web\\system\\sys_config.php";
    $conf = file_get_contents($file_name);
    $conf_data = explode("\n", $conf);
    $patch_replace = false;
    $i = 0;
    foreach ($conf_data as $key => $value) {
        //echo "replacing ";
        if (preg_match("/};/", $value) != 0){
            $patch_replace = false;
            continue;
        }
        if (preg_match("/change_http_port = function/", $value) != 0 || $patch_replace == true) {
            $patch_replace = true;
            if($i < count($patch)){
                echo "sysconfig repalce $patch[$i]";
                $conf_data[$key] = $patch[$i];
                $i++;
            }
            else {
                $conf_data[$key] = "\r\n";
            }
            continue;
        }
    }
    if($i == 0){
        return 1;
    }
    if (!copy($file_name, $file_name.".bak")) {
        echo "modify_apache, backup $file_name fail \n";
        return 1;
    }
    $file_data = "";
    foreach ($conf_data as $line) {
        $file_data .= $line."\r\n";
    }
    $tmp_name = getDcRoot()."/httpd-ssl_tmp.js"; // 临时文件
    if (!file_put_contents($tmp_name, $file_data)
        || !rename($tmp_name, $file_name)) {
        echo "modify_apache, save $file_name fail \n";
        unlink($tmp_name);
        return 1;
    }
    return 0;
}


function install_nginx($port){
    if($port == "14445"){
        return 1;
    }
    modify_apache("14445");
    modify_php_conf();
    $src_dir = __DIR__;
    $dst_dir = getDcRoot();
    $patches = array(
		"nginx",
        "stopall.bat",
        "startwebserver.exe",
        "startall.bat",
        "ldb/bin/mapreduce/app/web/system/sys_config.php"
    );
    if(modify_sys_config() == 0){
        array_pop($patches);
    }
    else {
        $md5_array = array("7781bc259904461d20bb8affa14bd7b8",
            "fa60ea7e3b8f51ab74aaad8a4c80a111",
            "095b1ab7e91f053239e1af6ce4d81237",
            "35678a04782406f61213486fb66f1032",
            "0952243a7833341d0667fd4e8569e42f",
            "5b2eca6bb7d484fbae80b54b7bebf4d3",
            "c6353318a6e73bc7b9ca1a73142d37bc");
        $file_md5 = md5_file($dst_dir."/ldb/bin/mapreduce/app/web/system/sys_config.php");
        if($file_md5 === false){
            echo "calculator md5 failed, check file sys_config.php\n";
            return 1;
        }
        foreach($md5_array as $value){
            if($file_md5 == $value){
                $file_md5 = 0;
                break;
            }
        }
        if($file_md5 != 0){
            echo "error file md5 sys_config.php is modified";
            return 1;
        }
    }
    foreach ($patches as $key => $var) {
		$src = "$src_dir/DataCenter/$var";
		$dst = "$dst_dir/$var";
		if (file_exists($dst)) {
			rename($dst,$dst.".bak");
		}
				
		if (is_dir($src)) {
			if (!system("xcopy /E /F /Y \"$src\" \"$dst/\"")) {
				echo "abort! xcopy $src to $dst failed!\n";
				return 1;
			}
		} else {
			if (!copy($src, $dst)) {
				echo "abort! cp $src to $dst failed!\n";
				return 1;
			}
		}
	}
    return 0;
}

function windows_modify_regedit($serv_path,$mode){
    if($mode == "install"){
        $websvr = "\"".$serv_path."\\startwebserver.exe\"";
    }
    else if($mode == "uninstall"){
        $websvr = "\"".$serv_path."\apache\bin\websvr.exe\""." -k runservice";
    }
    try { 
        $shell = new COM("WScript.Shell") or die("load WScript.Shell failed!");
        $shell->RegWrite("HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\Sangfor Web Server\\ImagePath", $websvr);
        echo("regedit websvr=$websvr\n");
        echo("modify regdit ok ...\n");
    } catch (Exception $e) {
        echo("modify regdit failed ...\n");
        return 1;
    }
    return 0;
}

function uninstall_nginx($port){
    $dst_dir = getDcRoot();
    $patches = array(
		"nginx",
        "apache/conf/extra/httpd-ssl.conf",
        "stopall.bat",
        "startwebserver.exe",
        "startall.bat",
        "ldb/bin/mapreduce/app/web/system/sys_config.php",
        "ldb/php/php.ini",
        "ldb/php/php-cgi-fcgi.ini"
	);
    foreach ($patches as $key => $var) {
		$dst = "$dst_dir/$var";
		if (file_exists($dst.".bak")) {
            unlink($dst);
            rename($dst.".bak",$dst);
            continue;
		}
				
		if (is_dir($dst)) {
			if (!ldb_rrmdir($dst)) {
				echo "don't care about this warning\n";
			}
		} else if (file_exists($dst)) {
			if (!unlink($dst)) {
				echo "don't care about this warning\n";
			}
		}
	}
    return 0;
}

function uninstall() {
    global $SP_NAME;
    
    //版本号还原
    $file_version = getDcRoot()."\\version\\appversion";
    $bak_version  = $file_version.".$SP_NAME";
    if(!file_exists($file_version) || !file_exists($bak_version)) {
        echo "$file_version or $bak_version not exists \n";
        return 1;
    }
    if(!rename($bak_version, $file_version)) {
        echo "rename $bak_version, $file_version fail \n";
        return 1;
    }
    if (uninstall_nginx(getDcRoot())) {
        echo "uninstall nginx failed\n";
        return 1;
    }
    if(windows_modify_regedit(getDcRoot(),"uninstall")){
        echo "abort! modify regedit failed\n";
        return 1;
    }
    return 0;
}

function install() {
	check_nginx();
    $config = file_get_contents(getDcRoot()."\\dc\\config\\sys_config.js");
    
    $sys_config = json_decode($config,true);
    $port = $sys_config["advance"]["http_port"];
    if(install_nginx($port)==1){
        echo "abort! 14445 port not support in this SP package";
        return 1;
    }
    if($port != "443"){
        if (renew_nginx_conf($port) == 1){
            echo "abort! modify nginx conf failed\n";
            return 1;
        }
    }
    if(windows_modify_regedit(getDcRoot(),"install")){
        echo "abort! modify regedit failed\n";
        return 1;
    }
    global $SP_NAME;
    update_version($SP_NAME);
    return 0;
}

// 执行主函数
function main($opr) {
    $build_time = getVersionBuildTime();
    if($build_time > "20220217"){
        echo "abort! version is not surport";
        return 1;
    }
    if ($opr === "install") {
		stopService(getDcRoot());
        $ret = install();
		if($ret == 1){
			exit($ret);
		}
        startService(getDcRoot());
        $dst_dir = getDcRoot();
        $WshShell = new COM("WScript.Shell");
        $WshShell->Run("\"$dst_dir\\startwebserver.exe\"", 3, true);
        exit($ret);
    } else if ($opr === "uninstall") {
        ldb_exec("taskkill /im ba_nginx.exe /f /t 1>nul 2>&1", $output, $ret);
		stopService(getDcRoot());
        $ret = uninstall();
		startService(getDcRoot());
		exit($ret);
    } else {
        echo "operate unsupport!\n";
        exit(1);
    }	
}

global $argv;
if (count($argv) < 2) {
    echo "operate unsupport!\n";
    exit(1);
}
main($argv[1]);
?>